package org.apache.tomcat.util.http.fileupload;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.File;
public class FileUtils {
    public static void deleteDirectory ( final File directory ) throws IOException {
        if ( !directory.exists() ) {
            return;
        }
        if ( !isSymlink ( directory ) ) {
            cleanDirectory ( directory );
        }
        if ( !directory.delete() ) {
            final String message = "Unable to delete directory " + directory + ".";
            throw new IOException ( message );
        }
    }
    public static void cleanDirectory ( final File directory ) throws IOException {
        if ( !directory.exists() ) {
            final String message = directory + " does not exist";
            throw new IllegalArgumentException ( message );
        }
        if ( !directory.isDirectory() ) {
            final String message = directory + " is not a directory";
            throw new IllegalArgumentException ( message );
        }
        final File[] files = directory.listFiles();
        if ( files == null ) {
            throw new IOException ( "Failed to list contents of " + directory );
        }
        IOException exception = null;
        for ( final File file : files ) {
            try {
                forceDelete ( file );
            } catch ( IOException ioe ) {
                exception = ioe;
            }
        }
        if ( null != exception ) {
            throw exception;
        }
    }
    public static void forceDelete ( final File file ) throws IOException {
        if ( file.isDirectory() ) {
            deleteDirectory ( file );
        } else {
            final boolean filePresent = file.exists();
            if ( !file.delete() ) {
                if ( !filePresent ) {
                    throw new FileNotFoundException ( "File does not exist: " + file );
                }
                final String message = "Unable to delete file: " + file;
                throw new IOException ( message );
            }
        }
    }
    public static void forceDeleteOnExit ( final File file ) throws IOException {
        if ( file.isDirectory() ) {
            deleteDirectoryOnExit ( file );
        } else {
            file.deleteOnExit();
        }
    }
    private static void deleteDirectoryOnExit ( final File directory ) throws IOException {
        if ( !directory.exists() ) {
            return;
        }
        directory.deleteOnExit();
        if ( !isSymlink ( directory ) ) {
            cleanDirectoryOnExit ( directory );
        }
    }
    private static void cleanDirectoryOnExit ( final File directory ) throws IOException {
        if ( !directory.exists() ) {
            final String message = directory + " does not exist";
            throw new IllegalArgumentException ( message );
        }
        if ( !directory.isDirectory() ) {
            final String message = directory + " is not a directory";
            throw new IllegalArgumentException ( message );
        }
        final File[] files = directory.listFiles();
        if ( files == null ) {
            throw new IOException ( "Failed to list contents of " + directory );
        }
        IOException exception = null;
        for ( final File file : files ) {
            try {
                forceDeleteOnExit ( file );
            } catch ( IOException ioe ) {
                exception = ioe;
            }
        }
        if ( null != exception ) {
            throw exception;
        }
    }
    public static boolean isSymlink ( final File file ) throws IOException {
        if ( file == null ) {
            throw new NullPointerException ( "File must not be null" );
        }
        if ( File.separatorChar == '\\' ) {
            return false;
        }
        File fileInCanonicalDir = null;
        if ( file.getParent() == null ) {
            fileInCanonicalDir = file;
        } else {
            final File canonicalDir = file.getParentFile().getCanonicalFile();
            fileInCanonicalDir = new File ( canonicalDir, file.getName() );
        }
        return !fileInCanonicalDir.getCanonicalFile().equals ( fileInCanonicalDir.getAbsoluteFile() );
    }
}
